<?php

namespace App\Connector\Models\Profile;

use App\Core\Models\BaseMongo;

class Model extends BaseMongo
{
    protected $table = 'profile';
    protected $isGlobal = true;

    public function createUpdateProfile($data, $overWriteExistingProducts = true)
    {
        $userId = $this->di->getUser()->id;
        if (isset($data['category_id'], $data['name'])) {
            $marketplace = $data['marketplace'] ?? 'cedcommerce';
            $data['marketplace'] = $marketplace;

            $data['user_id'] = $userId;
            $exists = $this->loadByField([
                "category_id" => $data['category_id'],
                "user_id" => $userId,
                "marketplace" => $marketplace,
            ]);
            if (isset($data['new_profile']) && $data['new_profile'] == true) {
                $exists = null;
            }

            if (isset($data['id'])) {
                $exists = $this->loadByField([
                    "_id" => $data['id'],
                    "user_id" => $userId,
                    "marketplace" => $marketplace,
                ]);
            }
            // $this->di->getLog()->logContent('Json: ' . json_encode($exists, 128), 'info', 'Response[Data].log');

            if ($exists) {
                if (isset($data['targets'], $exists['targets'])) {
                    $exists['targets'] = (array) $exists['targets'];
                    $data['targets'] = array_merge($exists['targets'], $data['targets']);
                } elseif (isset($exists['targets'])) {
                    $exists['targets'] = (array) $exists['targets'];
                    $data['targets'] = $exists['targets'];
                }

                $data['_id'] = $exists['_id'];
            }

            $eventsManager = $this->di->getEventsManager();

            $eventsManager->fire('profile:beforeSave', $this, ['custom_data' => &$data]);

            if (!empty($data['targets'])) {
                $targets = (array) $data['targets'];

                foreach ($targets as $targetMarketPlace => $value) {
                    if (!isset($data['targets'][$targetMarketPlace]['category_id'])) {
                        $catModel = new \App\Connector\Models\Category;
                        $mappedData = $catModel->loadByField(['marketplace_id' => $data['category_id']]);

                        if ($mappedData) {
                            if (!empty($mappedData['mapping'])) {
                                if (isset($mappedData['mapping'][$targetMarketPlace])) {
                                    $data['targets'][$targetMarketPlace]['category_id'] = $mappedData['mapping'][$targetMarketPlace];
                                    $data['targets'][$targetMarketPlace]['category_info'] = ['full_path' => $mappedData['full_path'] ?? '', 'category_id' => $mappedData['mapping'][$targetMarketPlace]];
                                } else {
                                    $savedData['targets'][$targetMarketPlace]['category_id'] = $data['category_id'];
                                }
                            } else {
                                $data['targets'][$targetMarketPlace]['category_id'] = $data['category_id'];
                            }
                        } else {
                            $data['targets'][$targetMarketPlace]['category_id'] = $data['category_id'];
                        }
                    }
                }
            }

            $this->setData($data);

            if ($this->save()) {
                $eventsManager->fire('profile:afterSave', $this, ['custom_data' => &$data]);
                $savedData = $this->getData();
                $savedData['id'] = (string) $savedData['_id'];
                unset($savedData['_id']);
                //Start save profile id in the conatainer
                $ruleExtract = $savedData;
                $ruleExtract['query'] = $ruleExtract['query'];

                $this->assignProfileToProducts($ruleExtract, $savedData, $overWriteExistingProducts);

                //end save

                return ['success' => true, 'message' => 'Data inserted successfully', 'data' => $savedData];
            } else {
                return ['success' => false, 'message' => 'something went wrong', 'code' => 'mongo_save_error'];
            }
        } else {
            return ['success' => false, 'message' => 'category_id or name missing', 'code' => 'data_missing'];
        }
    }

    public function assignProfileToProducts($ruleExtract, $savedData = [], $overWriteExistingProducts = false)
    {
        $helperObj = new Helper();
        $getAllRule = $helperObj->extractRuleFromProfileData($ruleExtract);
        if (!$getAllRule['success']) {
            return $getAllRule;
        }
        $customizedFilterQuery = $getAllRule['data'];
        if ($customizedFilterQuery) {
            $customizedFilterQuery['$and'][] = [
                'user_id' => [
                    '$eq' => $this->di->getUser()->id,
                ],
            ];

            if (!$overWriteExistingProducts) {
                $customizedFilterQuery['$and'][] = [
                    'profile' => ['$exists' => false],
                ];
            }
            $collection = $this->getCollectionForTable('product_container');
            $profileData = [];
            $profileData['profile_id'] = $savedData['id'];
            $profileData['profile_name'] = $savedData['name'];
            $profileData['app_tag'] = $this->di->getAppCode()->getAppTag();
            // $this->di->getLog()->logContent('Json: ' . json_encode($customizedFilterQuery),'info','ResponseFiletr.log');
            $collection->updateMany($customizedFilterQuery, ['$pull' => ['profile' => ['app_tag' => $profileData['app_tag']]]]);
            // $collection->updateMany($customizedFilterQuery, ['$pull'=>['profile'=>['app_tag'=>$profileData['app_tag']]]]);
            $collection->updateMany($customizedFilterQuery, ['$push' => ['profile' => $profileData]]);
            $collection->updatemany($customizedFilterQuery, [
                '$set' => [
                    'global_attributes' => true,
                ],
                '$unset' => [
                    'marketplace_attributes' => 1,
                ],
            ]);
        }
    }

    public function getProfile($data)
    {
        $userId = $this->di->getUser()->id;
        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('profile');
        if (isset($data['filters'])) {
            $data = $data['filters'];
            if (!isset($data['user_id'])) {
                $data['user_id'] = $userId;
            }

            $filterParams = [];
            $allwedFilters = $this->allwedFilters();

            foreach ($data as $key => $value) {
                if (isset($allwedFilters[$key])) {
                    if ($allwedFilters[$key] instanceof \Closure) {
                        if (call_user_func($allwedFilters[$key], $data)) {
                            $getData = call_user_func($allwedFilters[$key], $data);
                            $filterParams = array_merge($filterParams, $getData);
                        }
                    } else {
                        $filterParams[$key] = $value;
                    }
                } else {
                    $filterParams = [];
                    $returnArray = ['success' => false, 'code' => 'invalid filter(s)', 'message' => $key . ' is a invalid filter key'];
                    break;
                }
            }

            if (!empty($filterParams)) {
                $getProfile = $this->findByField($filterParams);
                return ['success' => true, 'data' => $getProfile ?? [], "count" => $collection->count($filterParams)];
            }
        } else {
            $getProfile = $this->findByField(['user_id' => $userId]);
            return ['success' => true, 'data' => $getProfile ?? [], "count" => $collection->count(['user_id' => $userId])];
        }
        return $returnArray;
    }

    public function allwedFilters()
    {
        return [
            'id' => function ($data) {
                return ['_id' => new \MongoDB\BSON\ObjectId($data['id'])];
            },
            'category_id' => 1,
            'marketplace' => function ($data) {
                return ['targets.' . $data['marketplace'] => ['$exists' => true]];
            },
            'name' => 1,
            'user_id' => 1,
        ];
    }

    public function allwedDeleteFilters()
    {
        return [
            'id' => function ($data) {
                return ['_id' => new \MongoDB\BSON\ObjectId((string) $data['id'])];
            },
            'user_id' => 1,
            // "marketplace"=>1

        ];
    }

    public function deleteProfile($data)
    {
        // $this->di->getLog()->logContent('$data: ' . json_encode($data, 128), 'info', 'ShubhLog.log');
        $deleteData = 0;
        $userId = $this->di->getUser()->id;
        $filterParams = [];
        $allwedFilters = $this->allwedDeleteFilters();
        if (!isset($data['user_id'])) {
            $data['user_id'] = $userId;
        }
        // $data['marketplace']='mercado_cbt';
        $data1 = [];
        foreach ($data as $key => $value) {
            if ($key != 'target_marketplace') {
                $data1[$key] = $data[$key];
            }
        }
        $data = $data1;
        foreach ($data as $key => $value) {
            if (isset($allwedFilters[$key])) {
                if ($allwedFilters[$key] instanceof \Closure) {
                    if (call_user_func($allwedFilters[$key], $data)) {
                        $getData = call_user_func($allwedFilters[$key], $data);
                        $filterParams = array_merge($filterParams, $getData);
                    } else {
                        $returnArray = ['success' => false, 'code' => 'invalid_value', 'message' => $key . ' value is invalid'];
                        break;
                    }
                } else {
                    $filterParams[$key] = $value;
                }
            } else {
                $filterParams = [];
                $returnArray = ['success' => false, 'code' => 'invalid_index', 'message' => $key . ' is a invalid  index'];
                break;
            }
        }

        //$this->di->getLog()->logContent('$filterparams: ' . json_encode($filterParams, 128), 'info', 'ShubhLog.log');
        // return ["success" => true];
        if (!empty($filterParams)) {
            // print_r($filterParams);die;
            $eventsManager = $this->di->getEventsManager();

            $eventsManager->fire('profile:beforeDelete', $data);

            if ($this->getCollection()->deleteOne($filterParams, ['w' => true])) {
                $deleteData++;
            }
            $collection = $this->getCollectionForTable('product_container');
            $collection->updateMany([
                "user_id" => $userId,
                "profile_id" => $data["id"],
            ], [
                '$unset' => [
                    "profile" => "",
                ],
            ]);
            $eventsManager->fire('profile:afterDelete', $data);
            $returnArray = ['success' => true, 'message' => 'Data deleted successfully', 'data' => ['deleteData' => $deleteData]];
        }
        return $returnArray;
    }
}
